/*
  $Id: Location.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.HashMap;
import java.util.Map;

/**
	This enumeration defines constants that are used to specify different
	file storage locations.

	@author enell
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
 */
public enum Location
{
	/**
		The item is offline and doesn't use any quota.
	*/
	OFFLINE(0, "Offline", false, false),
	/**
		The item is stored in primary storage.
	*/
	PRIMARY(1, "Primary", true, true),
	/**
		The item is stored in secondary storage. Note! This type
		of storage may not be available on all servers.
	*/
	SECONDARY(2, "Secondary", false, true),
	/**
		The item is stored externally. Some information like
		size, md5, etc. may not be available.
		@since 2.16
	*/
	EXTERNAL(3, "External", true, false);
	
	/**
		Maps an integer to a location.
	*/
	private static final Map<Integer, Location> valueMapping = new HashMap<Integer, Location>();
	
	static
	{
		for(Location location : Location.values())
		{
			Location l = valueMapping.put(location.getValue(), location);
			assert l == null : "Another location with the value "+location.getValue()+" already exists";
		}
	}
	
	/**
		The integer value of this location. 
	*/
	private final int value;
	/**
		Returned by the toString method.
	*/
	private final String displayValue;

	/**
		If the file can be downloaded from this location.
	*/
	private final boolean downloadable;

	/**
		If this location is limited by quota.
	*/
	private final boolean quotaLimited;

	/**
	 	Creates a new location. Used internal only.
	 	
		@param value The integer value of this location.
	*/
	private Location(int value, String displayValue, boolean downloadable, boolean quotaLimited)
	{
		this.value = value;
		this.displayValue = displayValue;
		this.downloadable = downloadable;
		this.quotaLimited = quotaLimited;
	}
	
	@Override
	public String toString()
	{
		return displayValue;
	}
	
	/**
		Get the <code>Location</code> object when you know the integer code.
		
		@param value The integer value.
		@return The location for the integer value.
	*/
	public static Location fromValue(int value)
	{
		Location location = valueMapping.get(value);
		assert location != null : "location == null for value "+value;
		return location;
	}
	
	/**
		Get the integer value that is used when storing an location to the database.
		
		@return The integer value for this location.
	*/
	public int getValue()
	{
		return value;
	}
	
	/**
		A flag indicating if a file that is stored at this location can
		be downloaded by a call to {@link File#getDownloadStream(long)}
		or {@link File#download(java.io.OutputStream, long)}.
		@return TRUE if the file can downloaded, FALSE if not
		@since 2.16
	*/
	public boolean isDownloadable()
	{
		return downloadable;
	}
	
	/**
		A flag indicating if a file that is stored at this location
		should be included in quota limit.
		@return TRUE if this location is quota limited, FALSE otherwise
		@since 2.16
	*/
	public boolean isQuotaLimited()
	{
		return quotaLimited;
	}

}
