/*
	$Id: StringUtil.java 6022 2012-03-22 14:01:20Z nicklas $

	Copyright (C) 2005, 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collection;

/**
	Utility methods that will make it easier to implement
	data validation for string attributes.
	
	@author Nicklas
	@version 2.0
*/
public class StringUtil
{

	/**
		Trim and check the length of a string. Null values are allowed.

		@param value The string to check
		@param name The name of the attribute to use if an exception is thrown
		@param maxLength The maximum length of the string
		@return The original string, trimmed from white-space at the 
			beginning and end
		@throws StringTooLongException If the string is too long
	*/
	public static String setNullableString(String value, String name, int maxLength)
		throws StringTooLongException
	{
		if (value == null) return null;
		value = value.trim();
		if (value.length() > maxLength) throw new StringTooLongException(name, value, maxLength);
		return value;
	}
	
	/**
		Trim and check the length of a string. Null values are not allowed.

		@param value The string to check
		@param name The name of the attribute to use if an exception is thrown
		@param maxLength The maximum length of the string
		@return The original string, trimmed from white-space at the 
			beginning and end
		@throws StringTooLongException If the string is too long
		@throws InvalidUseOfNullException If the string is null
	*/
	public static String setNotNullString(String value, String name, int maxLength)
		throws InvalidUseOfNullException, StringTooLongException
	{
		if (value == null) throw new InvalidUseOfNullException(name);
		value = value.trim();
		if (value.length() > maxLength) throw new StringTooLongException(name, value, maxLength);
		return value;
	}
	
	/**
		Check if two strings are equal or both are null.
		@param s1 The first string
		@param s2 The second string
		@return TRUE if the strings are equal or both are null, FALSE otherwise
	*/
	public static boolean isEqualOrNull(String s1, String s2)
	{
		return (s1 == s2) || (s1 != null &&  s1.equals(s2));
	}
	
	/**
		Check if two strings are equal or both are null.
		@param s1 The first string
		@param s2 The second string
		@param caseInsensitive TRUE if the comparison should ignore case, FALSE
			otherwise
		@return TRUE if the strings are equal or both are null, FALSE otherwise
		@since 2.4
	*/
	public static boolean isEqualOrNull(String s1, String s2, boolean caseInsensitive)
	{
		if (!caseInsensitive)
		{
			return isEqualOrNull(s1, s2);
		}
		else
		{
			return (s1 == s2) || (s1 != null &&  s1.equalsIgnoreCase(s2));
		}
	}
	
	/**
	 	Join a collection of values to a string, using the specified delimiator.
		@param values The values to join
		@param deliminator The deliminator between the values
		@param skipNull If null values should be skipper or not
		@return The resulting string
	*/
	public static final String join(Collection<?> values, String deliminator, boolean skipNull)
	{
		StringBuilder sb = new StringBuilder();
		boolean firstElement = true;
		if (values != null)
		{
			for (Object value : values)
			{
				if (value == null)
				{					
					if (skipNull)
					{
						/* #### CONTINUE-STATEMENT #### */
						continue;
					}
					else
					{
						value = "";
					}
				}
				if (!firstElement) sb.append(deliminator);
				sb.append(value);
				firstElement = false;
			}
		}
		return sb.toString();
	}

	/**
		Trim a string to a maximum length. String longer than the specified length
		will be cut off and three dots are appended. If no trimming is needed the
		original string is returned.
		@param s The string to trim
		@param length The maximum length of the string
		@return The trimmed string
	*/
	public static final String trimString(String s, int length)
	{
		if (s != null && s.length() > length)
		{
			s = s.substring(0, length - 3) + "...";
		}
		return s;
	}
	
	/**
		Trim a string to a maximum length. Strings longer than the specified 
		length will have text cut-out from the middle and replaced by three 
		dots. If no trimming is needed the original string is returned.
		@param s The string to trim
		@param length The maximum length of the string
		@return The trimmed string
		@since 2.14
	*/
	public static final String trimStringMiddle(String s, int length)
	{
		if (s != null && s.length() > length)
		{
			int charactersToCut = s.length() + 3 - length;
			int beginAt = (s.length() - charactersToCut) / 2;
			s = s.substring(0, beginAt) + "..." + s.substring(beginAt + charactersToCut);
		}
		return s;
	}
	
	/**
		Tokenize a string by splitting it at positions matching the
		regular expression, optionally ignoring matches that are inside
		quotes.
		
		@param s The string to tokenize
		@param regexp The regular expression for matching split sites
		@param notInsideQuotes If TRUE, the regexp is not matched inside quotes
		@return An array with the elements of the array
		@since 2.15
	*/
	public static final String[] tokenize(String s, String regexp, boolean notInsideQuotes)
	{
		return tokenize(s, regexp, notInsideQuotes, false);
	}

	/**
		Tokenize a string by splitting it at positions matching the
		regular expression. Matches that are inside quotes can optionally
		be ignored. Elements with quotes can optionally have the quotes
		removed.
		
		@param s The string to tokenize
		@param regexp The regular expression for matching split sites
		@param notInsideQuotes If TRUE, the regexp is not matched inside quotes
		@param removeQuotes If TRUE, remove quotes from all elements before returning the
			result 
			
		@return An array with the elements of the array
		@since 2.17
	*/
	public static final String[] tokenize(String s, String regexp, boolean notInsideQuotes, boolean removeQuotes)
	{
		String[] result = null;
		if (s != null && regexp != null)
		{
			if (notInsideQuotes) regexp += "(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))";
			result = s.split(regexp);
			if (removeQuotes)
			{
				for (int i = 0; i < result.length; ++i)
				{
					String t = result[i];
					if (t != null && t.startsWith("\"") && t.endsWith("\""))
					{
						result[i] = t.substring(1, t.length()-1);
					}
				}
			}
		}
		return result;
	}


	/**
		Find the longest common prefix for a collection of strings. If
		the collection is null, empty or contains at least one null
		element null is returned. If the given strings doesn't have
		a common prefix the empty string is returned.
		
		@param strings A collection of string objects
		@return The longes string such that {@link String#startsWith(String)} 
			returns true for all strings in the given collection
		@since 2.17
	*/
	public static final String getCommonPrefix(Collection<String> strings)
	{
		if (strings == null || strings.isEmpty() || strings.contains(null)) return null;
		// Early exit if the collection only contains a single element
		if (strings.size() == 1) return strings.iterator().next();
		
		// Copy to temporary array for easier iteration
		String[] tmp = new String[strings.size()];
		strings.toArray(tmp);
		
		// This is the length of the common prefix so far...
		int prefixLength = 0;
		
		// Use the first string as the reference string.
		String ref = tmp[0];
		
MAIN:
		while (prefixLength < ref.length())
		{
			char refChar = ref.charAt(prefixLength);
			// Test if all strings have the same character at the given position
			for (int i = 1; i < tmp.length; ++i)
			{
				String s = tmp[i];
				if (prefixLength >= s.length() || refChar != s.charAt(prefixLength))
				{
					break MAIN;
				}
			}
			// All strings have the same character at index 'prefixLength'
			// Move on to the next position
			prefixLength++;
		}
		return ref.substring(0, prefixLength);
		
	}
	
	/**
		Decapitalize the string by making sure the first character is lower-case. All
		other characters are left as they are. If the string is empty, the empty string
		is returned. If the string is null, null is returned.
		@since 3.2
	*/
	public static final String decapitalize(String value)
	{
		if (value == null || value.length() == 0) return value;
		return value.substring(0, 1).toLowerCase() + value.substring(1);
	}
}
