/*
	$Id: QuotaIndex.java 6468 2014-06-10 11:27:02Z nicklas $

	Copyright (C) 2005 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Set;

/**
	This is a helper class for the {@link QuotaData} item to
	help with the Hibernate mapping between quota and
	quota type/location/max bytes. This class holds quota type/location
	pairs used as keys in the {@link QuotaData#getQuotaValues()}
	map.

	@author enell
	@version 2.0
	@see <a href="../../../../../../html/developer/api/data_api.quota.html">Developer documentation: Quota and disk usage</a>
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (ti, 10 jun 2014) $
*/
public class QuotaIndex
	implements Serializable
{
	private static final long serialVersionUID = 4679331005994321116L;

	
	QuotaIndex()
	{}
	
	public QuotaIndex(QuotaTypeData quotaType, int location)
	{
		this.quotaType = quotaType;
		this.location = location;
	}

	private int location;

	/**
	 	@hibernate.property column="`location`" type="int"
	*/
	public int getLocation()
	{
		return location;
	}

	void setLocation(int location)
	{
		this.location = location;
	}

	private transient QuotaTypeData quotaType;

	/**
		@hibernate.many-to-one column="`quotaType_id`" outer-join="false"
	*/
	QuotaTypeData getQuotaType()
	{
		return quotaType;
	}

	void setQuotaType(QuotaTypeData quotaType)
	{
		this.quotaType = quotaType;
	}

	private long maxBytes;
	/**
	 	@hibernate.property column="`max_bytes`" type="long"
	*/
	public long getMaxBytes()
	{
		return maxBytes;
	}
	
	public void setMaxBytes(long maxBytes)
	{
		this.maxBytes = maxBytes;
	}

	/**
		Find the quota index entry in the given set and return the 'maxBytes' value.
		NOTE! Used to be stored in a Map, but due to changes between Hibernate 3 and
		Hibernate 4 this seems to no longer work and we need to do it like this instead.
		@since 3.4
	*/
	public Long findMaxBytes(Set<QuotaIndex> quota)
	{
		if (quota == null) return null;
		for (QuotaIndex qi : quota)
		{
			if (this.equals(qi)) 
			{
				return qi.getMaxBytes();
			}
		}
		return null;
	}
	
	@Override
	public String toString()
	{
		return this.quotaType.getName() + ":" + this.location + ":" + this.maxBytes;
	}
	
	/**
		Check if this object is equal to another <code>QuotaIndex</code>
		object. They are equal if both have the same quotatype id and location.
	*/
	@Override
	public boolean equals(Object o)
	{
		if ((o == null) || (getClass() != o.getClass()))
		{
			return false;
		}
		QuotaIndex qi = (QuotaIndex) o;
		return (this.quotaType.getId() == qi.quotaType.getId()) && (this.location == qi.location);
	}

	/**
		Calculate the hash code for the object.
	*/
	@Override
	public int hashCode()
	{
		return (int) (53 * quotaType.getId() + 17 * location);
	}
	
	private void readObject(ObjectInputStream ois)
		throws ClassNotFoundException, IOException 
	{
		ois.defaultReadObject();
		int quotaTypeId = ois.readInt();
		quotaType = new QuotaTypeData();
		quotaType.setId(quotaTypeId);
	}
	private void writeObject(ObjectOutputStream ois)
		throws IOException 
	{
		ois.defaultWriteObject();
		ois.writeInt(quotaType.getId());
	}

}
