/**
	$Id: PostgresDbEngine.java 6684 2015-01-14 13:29:04Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.dbengine;

import java.util.Collection;
import java.util.Set;

import net.sf.basedb.core.hibernate.SchemaGenerator;

import org.hibernate.dialect.Dialect;
import org.hibernate.mapping.PersistentClass;

/**
	Database engine for Postgres.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2015-01-14 14:29:04 +0100 (on, 14 jan 2015) $
*/
public class PostgresDbEngine
	extends AbstractDbEngine
	implements DbEngine
{

	/**
		Create PostgresDbEngine.
	*/
	public PostgresDbEngine()
	{}

	/*
		From the DbEngine interface
		-------------------------------------------
	*/
	/**
		Always false.
	*/
	@Override
	public boolean useThetaJoin()
	{
		return false;
	}
	
	/**
		Generate <code>CREATE SCHEMA &lt;schema&gt;</code>.
		The catalog parameter is not supported.
		@since 3.4
	*/
	@Override
	public String getCreateSchemaSql(String catalog, String schema)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("CREATE SCHEMA ").append(getQuotedName(schema));
		return sql.toString();
	}
	
	/**
		Generate <code>ALTER TABLE &lt;schema&gt;.&lt;table&gt; ADD CONSTRAINT &lt;name&gt;
		UNIQUE (&lt;columns&gt)</code> for a unique index, 
		<code>CREATE INDEX &lt;name&gt; ON &lt;schema&gt;.&lt;table&gt; (&lt;columns&gt;)</code>
		for other indexes.
		The catalog parameter is not supported.
	*/
	@Override
	public String getCreateIndexSql(String catalog, String schema, String table, String name, 
		Set<String> columns, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		if (unique)
		{
			sql.append("ALTER TABLE ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" ADD CONSTRAINT ");
			sql.append(getQuotedName(name)).append(" UNIQUE (");
			appendList(sql, columns);
			sql.append(")");
		}
		else
		{
			sql.append("CREATE INDEX ");
			sql.append(getQuotedName(name)).append(" ON ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" (");
			appendList(sql, columns);
			sql.append(")");
			
		}
		return sql.toString();
	}
	
	/**
		Generate <code>ALTER TABLE &lt;schema&gt;.&lt;table&gt; DROP CONSTRAINT &lt;name&gt;</code>
		for unique indexes, <code>DROP INDEX &lt;schema&gt;.&lt;name&gt;</code> for other indexes.
		The catalog parameter is not supported.
	*/
	@Override
	public String getDropIndexSql(String catalog, String schema, String table, String name, boolean unique)
	{
		StringBuilder sql = new StringBuilder();
		if (unique)
		{
			sql.append("ALTER TABLE ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(table)).append(" DROP CONSTRAINT ");
			sql.append(getQuotedName(name));
		}
		else
		{
			sql.append("DROP INDEX ");
			if (schema != null) sql.append(getQuotedName(schema)).append(".");
			sql.append(getQuotedName(name));
		}
		return sql.toString();
	}

	/**
		Generate <code>ALTER TABLE schema.table ADD CONSTRAINT name
		FOREIGN KEY (columns) REFERERENCES refTable (refColumns)</code>
		@since 3.1
	 */
	@Override
	public String getCreateForeignKeySql(String catalog, String schema,	String table, 
		String name, Set<String> columns, String refTable, Set<String> refColumns) 
	{
		StringBuilder sql = new StringBuilder();
		
		sql.append("ALTER TABLE ");
		if (schema != null) sql.append(getQuotedName(schema)).append(".");
		sql.append(getQuotedName(table));
		sql.append(" ADD CONSTRAINT ");
		sql.append(getQuotedName(name));
		sql.append(" FOREIGN KEY (");
		appendList(sql, columns);
		sql.append(") REFERENCES ");
		sql.append(getQuotedName(refTable));
		sql.append(" (");
		appendList(sql, refColumns);
		sql.append(")");
		return sql.toString();
	}

	/**
		Generate <code>ALTER TABLE &lt;schema&gt;.&lt;table&gt; DROP CONSTRAINT &lt;name&gt;</code>
		@since 3.0
	*/
	@Override
	public String getDropForeignKeySql(String catalog, String schema, String table, String name)
	{
		return getDropIndexSql(catalog, schema, table, name, true);
	}
	
	
	/**
		Generate <code>ALTER TABLE schema.table ADD CONSTRAINT name
		PRIMARY KEY (columns)</code>.
		@since 3.1
	*/
	@Override
	public String getCreatePrimaryKeySql(String catalog, String schema, String table, String name, 
		Set<String> columns)
	{
		StringBuilder sql = new StringBuilder();
		sql.append("ALTER TABLE ");
		if (schema != null) sql.append(getQuotedName(schema)).append(".");
		sql.append(getQuotedName(table)).append(" ADD CONSTRAINT ");
		sql.append(getQuotedName(name)).append(" PRIMARY KEY (");
		appendList(sql, columns);
		sql.append(")");
		return sql.toString();
	}

	/**
		Generate <code>ALTER TABLE schema.table DROP CONSTRAINT name</code>
		@since 3.1
	*/
	@Override
	public String getDropPrimaryKey(String catalog, String schema, String table, String name) 
	{
		return getDropIndexSql(catalog, schema, table, name, true);
	}

	/**
		@return Always "nextval('sequenceName')" (assuming that the ID column is generated by a sequence)
		@since 3.0
	*/
	@Override
	public String getInsertAutoIncrementSql(PersistentClass pc)
	{
		// For sequence generators we must use: 
		// INSERT INTO ... (id, ...) VALUES (nextval('sequenceName'), ...)
			
		// The sequence name is found in the pc.identifierProperty.identifierGeneratorProperties
		org.hibernate.mapping.SimpleValue idValue = (org.hibernate.mapping.SimpleValue)pc.getIdentifierProperty().getValue();
		String sequenceName = idValue.getIdentifierGeneratorProperties().getProperty(org.hibernate.id.SequenceGenerator.SEQUENCE, "hibernate_sequence");
		return "nextval('" + sequenceName + "')";
	}

	/**
		Return true. Otherwise queries may take a very long time since
		indexes may not be used properly.
	*/
	@Override
	public boolean analyzeAfterBatchOperation()
	{
		return true;
	}

	/**
		Generate <code>VACUUM FULL &lt;schema&gt;.&lt;table&gt;</code>.
		The catalog parameter is not supported.
	*/
	@Override
	public String getOptimizeTableSql(String catalog, String schema, String table)
	{
		schema = schema == null ? "" : getQuotedName(schema) + ".";
		return "VACUUM FULL " + schema + getQuotedName(table);
	}
	
	/**
		Generate <code>ANALYZE &lt;schema&gt;.&lt;table&gt;</code>.
		The catalog parameter is not supported.
	*/
	@Override
	public String getAnalyzeTableSql(String catalog, String schema, String table)
	{
		schema = schema == null ? "" : getQuotedName(schema) + ".";
		return "ANALYZE " + schema + getQuotedName(table);
	}
	/**
		Returns FALSE.
		@since 2.4
	*/
	@Override
	public boolean caseInsensitiveComparison()
	{
		return false;
	}
	/**
		Returns FALSE.
	*/
	@Override
	public boolean supportsColumnAliasInWhere()
	{
		return false;
	}
	/**
		Returns TRUE.
	*/
	@Override
	public boolean supportColumnAliasInOrderBy()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	@Override
	public boolean supportColumnAliasInGroupBy()
	{
		return true;
	}
	/**
		Returns FALSE.
	*/
	@Override
	public boolean supportColumnAliasInHaving()
	{
		return false;
	}
	/**
		Returns TRUE.
	*/
	@Override
	public boolean checkForInvalidNumberOperation()
	{
		return true;
	}
	/**
		Returns TRUE.
	*/
	@Override
	public boolean selectOrderByColumnsIfDistinct()
	{
		return true;
	}
	/**
		Return <code>&lt;value&gt; ~ &lt;regexp&gt;</code>.
		@since 2.8
	*/
	@Override
	public String rlike(String value, String regexp)
	{
		return value + " ~ " + regexp;
	}
	/**
		Skip all "create index" statements since Hibernate doesn't generate those
		with unique names. PostgreSQL users should always run the ./updateindexes.sh
		tool which will create the missing indexes.
		@since 3.1
	*/
	@Override
	public String inspectSchemaGenerationSQL(String sql, Dialect dialect,
			SchemaGenerator.Mode mode)
	{
		sql = super.inspectSchemaGenerationSQL(sql, dialect, mode);
		if (sql != null && sql.startsWith("create index"))
		{
			sql = null;
		}		
		return sql;
	}
	/**
		@return Always TRUE
	 */
	@Override
	public boolean useSavePointToContinueTransactionFromSqlFailure() 
	{
		return true;
	}
	
	@Override
	public String getApproximateRowCountSql(String catalog, String schema, String table)
	{
		return "select reltuples from pg_class where relname = '" + table + "'";
	}
	// -------------------------------------------


	private void appendList(StringBuilder sb, Collection<String> items)
	{
		int i = 0;
		for (String column : items)
		{
			if (i > 0) sb.append(", ");
			sb.append(getQuotedName(column));
			i++;
		}
	}
	

}
