/**
	$Id: SnapshotManager.java 6857 2015-04-14 11:49:15Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.snapshot;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.Application;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Permission;
import net.sf.basedb.util.StaticCache;
import net.sf.basedb.util.filter.Filter;
import net.sf.basedb.util.filter.StaticFilter;

/**
	Manager for annotation snapshots. The manager keeps an internal cache in memory
	of snapshots that has been loaded. If an unload snapshot is needed the manager
	will first look in the static cache. If it is not found there the last resort
	is to generate a new snapshot.
	
	@author Nicklas
	@version 2.14
	@base.modified $Date: 2015-04-14 13:49:15 +0200 (ti, 14 apr 2015) $
*/
public class SnapshotManager
{

	private final Map<Integer, AnnotationSetSnapshot> snapshots;
	private DisabledInheritanceFilter disabledInheritanceFilter;
	
	/**
		The static cache key under which the snapshot for the
		annotation set with the given id is cached.
		@param annotationSetId The ID of an annotation set
	*/
	public static String getCacheKey(int annotationSetId)
	{
		int subdir = annotationSetId % 1000;
		return "/snapshots-v" + AnnotationSetSnapshot.FILE_VERSION +
			"/" + subdir + "/annotation-set-" + annotationSetId + ".ser";
	}
	
	/**
		Removes a snapshot from the static cache.
		@param annotationSetId The ID of the annotation set  
		@return TRUE if the snapshot was removed or didn't
			exists to begin with, FALSE if it existed and could
			not be removed
	*/
	public static boolean removeSnapshot(int annotationSetId)
	{
		StaticCache cache = Application.getStaticCache();
		String cacheKey = getCacheKey(annotationSetId);
		return cache.delete(cacheKey, 1000);
	}
	
	/**
		Removes a list of snapshots from the static cache.
		@param annotationSetIds A list with IDs of annotation set  s
		@return The number of successfully removed snapshots
		@since 3.5
	*/
	public static int removeSnapshots(List<Integer> annotationSetIds)
	{
		StaticCache cache = Application.getStaticCache();
		int removed = 0;
		for (Integer i : annotationSetIds)
		{
			String cacheKey = getCacheKey(i);
			if (cache.delete(cacheKey, 1000)) removed++;
		}
		return removed;
	}
	
	/**
		Create a new snapshot manager.
	*/
	public SnapshotManager()
	{
		this.snapshots = new HashMap<Integer, AnnotationSetSnapshot>();
	}
	
	/**
		Get the annotation snapshot for the annotation set with the given id.
		@param dc A DbControl to use if the snapshot has to be created from
			the database cache
		@param annotationSetId The id of the annotation set
		@return A snapshot, or null if the annotation set with the given
		 id doesn't exists
	*/
	public AnnotationSetSnapshot getSnapshot(DbControl dc, int annotationSetId)
	{
		// First check the internal cache
		AnnotationSetSnapshot snapshot = snapshots.get(annotationSetId);
		if (snapshot != null) return snapshot;
		
		// Then check the static cache
		StaticCache cache = Application.getStaticCache();
		String cacheKey = getCacheKey(annotationSetId);
		snapshot = (AnnotationSetSnapshot)cache.load(cacheKey, 1000);
		if (snapshot == null) 
		{
			// As a last resort, create a new snapshot and cache it
			snapshot = AnnotationSet.createSnapshot(dc, annotationSetId);
			if (snapshot != null) cache.store(cacheKey, snapshot, 1000);
		}
		snapshots.put(annotationSetId, snapshot);
		return snapshot;
	}

	/**
		Find annotations. Alwasys search primary and optionally inherited.
		@see #findAnnotations(DbControl, AnnotationSetSnapshot, Filter, boolean, boolean)
	*/
	public List<AnnotationSnapshot> findAnnotations(DbControl dc, AnnotationSetSnapshot snapshot, 
			Filter<? super AnnotationSnapshot> filter, boolean searchInherited)
	{
		return findAnnotations(dc, snapshot, filter, true, searchInherited);
	}

	/**
		Search in a snapshot for all annotations matching a given filter.
		If the snapshot inherits annotations and/or annotation sets from other
		annotations sets, those snapshots are search recursively one level down.
		
		@param dc A DbControl to use for database access if a snapshot doesn't
			exists for an inherited annotation set
		@param snapshot The snapshot to search
		@param filter A filter that should match the wanted annotation snapshots
			or null to match all annotations
		@param searchPrimary TRUE if primary annotations should be searched
		@param searchInherited TRUE if inherited annotations should be searched
		@return A list with the found annotations
		@since 3.5
	*/
	public List<AnnotationSnapshot> findAnnotations(DbControl dc, AnnotationSetSnapshot snapshot, 
			Filter<? super AnnotationSnapshot> filter, boolean searchPrimary, boolean searchInherited)
	{
		List<AnnotationSnapshot> result = new ArrayList<AnnotationSnapshot>();
		if (!searchPrimary && !searchInherited || snapshot == null) return result;
		
		if (filter == null) filter = new StaticFilter<AnnotationSnapshot>(true);

		// Needed for recursive searching of inherited annotations
		AnnotationFilter annotationFilter = null;
		DisabledInheritanceFilter recursiveFilter = null;
		if (searchInherited)
		{
			if (disabledInheritanceFilter == null) disabledInheritanceFilter = new DisabledInheritanceFilter(dc);
			recursiveFilter = new DisabledInheritanceFilter(disabledInheritanceFilter);
			annotationFilter = new AnnotationFilter();
		}
		
		for (AnnotationSnapshot shot : snapshot.getAnnotations())
		{
			if (shot.isInherited())
			{
				if (!searchInherited) continue; // with the next snapshot in the list
				
				int annotationId = shot.getAnnotationId();
				// If this is an inherited annotation set (annotationId==0), or if 
				// it the filter matches we must recursively load the inherited annotations
				if (annotationId == 0 || filter.evaluate(shot))
				{
					AnnotationSetSnapshot recursive = getSnapshot(dc, shot.getAnnotationSetId());
					if (recursive != null)
					{
						if (annotationId != 0) 
						{
							// The main filter has already matched this annotation
							// Use the recursive filter to load the specific annotation only
							annotationFilter.setAnnotationId(annotationId);
							recursiveFilter.setParentFilter(annotationFilter);
						}
						else
						{
							recursiveFilter.setParentFilter(filter);
						}
						
						for (AnnotationSnapshot primary : findAnnotations(dc, recursive, recursiveFilter, false))
						{
							AnnotationSnapshot copy = new AnnotationSnapshot();
							copy.initInherited(primary);
							result.add(copy);
						}
					}
				}
			}
			else
			{
				// Primary annotation
				if (!searchPrimary) continue; // with the next snapshot in the list
				
				if (filter.evaluate(shot) && shot.hasPermission(dc, Permission.READ)) 
				{
					shot.setItem(snapshot.getItemId(), snapshot.getItemType());
					result.add(shot);
				}
			}
		}
		return result;
	}
	
	/**
		@see #findAnnotations(DbControl, Annotatable, Filter, boolean, boolean)
	*/
	public List<AnnotationSnapshot> findAnnotations(DbControl dc, Annotatable item, 
			Filter<? super AnnotationSnapshot> filter, boolean searchInherited)
	{
		return findAnnotations(dc, item, filter, true, searchInherited);
	}
	
	/**
		Utility method for calling <code>getSnapshot</code> and <code>findAnnotations</code>
		in one go.
		@return A list with the annotation snapshots (an empty list is returned if the
			item doesn't have any annotations)
		@see #getSnapshot(DbControl, int)
		@see #findAnnotations(DbControl, AnnotationSetSnapshot, Filter, boolean, boolean)
		@since 3.5
	*/
	public List<AnnotationSnapshot> findAnnotations(DbControl dc, Annotatable item, 
		Filter<? super AnnotationSnapshot> filter, boolean searchPrimary, boolean searchInherited)
	{
		if (!item.isAnnotated()) return Collections.emptyList();
		AnnotationSetSnapshot snapshot = getSnapshot(dc, item.getAnnotationSet().getId());
		return findAnnotations(dc, snapshot, filter, searchPrimary, searchInherited);
	}

}
