/**
	$Id: InputStreamTracker.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.io.IOException;
import java.io.InputStream;

/**
	This class is used to keep track of the number of bytes read from the
	underlying input stream. This is very useful for code that needs to 
	reporter progress but are having difficulties to know the number of
	processed bytes.

	@author nicklas
	@version 2.4
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (to, 11 sep 2008) $
*/
public class InputStreamTracker
	extends InputStream
{

	/**
		The source input stream to read from.
	*/
	private final InputStream in;
	

	/**
		The number of bytes that has been read from the underlying 
		input stream so far.
	*/
	private long numRead;
	
	/**
		The number of bytes that had been read when {@link #mark(int)}
		was called.
	*/
	private long numReadAtMark;
	
	/**
		Create a new input stream tracker.
		@param in The input stream to read from
	*/
	public InputStreamTracker(InputStream in)
	{
		if (in == null) throw new NullPointerException("in");
		this.in = in;
	}
	/*
		From the InputStream class
		-------------------------------------------
	*/
	@Override
	public int available() 
		throws IOException
	{
		return in.available();
	}
	@Override
	public void close() 
		throws IOException
	{
		in.close();
	}
	@Override
	public synchronized void mark(int readlimit)
	{
		in.mark(readlimit);
		this.numReadAtMark = numRead;
	}
	@Override
	public boolean markSupported()
	{
		return in.markSupported();
	}
	@Override
	public int read() 
		throws IOException
	{
		int result = in.read();
		if (result != -1) ++numRead;
		return result;
	}
	@Override
	public int read(byte[] b, int off, int len) 
		throws IOException
	{
		int result = in.read(b, off, len);
		if (result > 0) numRead += result;
		return result;
	}
	@Override
	public int read(byte[] b) 
		throws IOException
	{
		return read(b, 0, b.length);
	}
	@Override
	public synchronized void reset() 
		throws IOException
	{
		in.reset();
		numRead = numReadAtMark;
	}
	@Override
	public long skip(long n)
		throws IOException
	{
		long result = in.skip(n);
		if (result > 0) numRead += result;
		return result;
	}
	// -------------------------------------------

	/**
		Get the number of bytes read or skipped from the input stream.
		If the underlying stream supports {@link #reset()} this will also
		reset the number of read bytes to the latest {@link #mark(int)}
		position.
		@return The number of bytes that has been read so far
	*/
	public long getNumRead()
	{
		return numRead;
	}

}
