/**
	$Id: PathConverter.java 5946 2012-02-03 13:18:47Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.xml;

import java.lang.reflect.Method;

/**
	A value converter that can be used on all setter methods
	that has been annotated with the {@link PathSetter} and
	{@link PathType} annotations. The values are converted using the 
	following rules:
	
	<ul>
	<li>If the value starts with '/', the root path is added to the start
		of the value, for example, <code>/images/copy.png --&gt; /base2/images/copy.png</code>.
	<li>If the value starts with '~', the path to the home directory for
		the current extension is added, for example,
		<code>~/images/myimage.png --&gt; /base2/extensions/my-extensions.jar/images/myimage.png</code>
	<li>If the path type is {@link PathType#CONTEXT_RELATIVE} the context-path is
		removed from the final path. Eg. <code>/base2/images/copy.png --&gt; /images/copy.png</code>
	</ul>

	<p>
	Note 1! It is only the start of the value that is checked. If a '~' is
	present in the middle of a value, it is not replaced.

	<p>
	Note 2! Using the {@link VariableSetter} annotation and values like
	<code>$ROOT$/images/copy.png</code> would have the same result. The only
	reason to use this annotation is that it allows for a shorter notation.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class PathConverter 
	implements ValueConverter
{

	private String root;
	private String home;
	/**
		Create a new empty path converter. Use {@link #setRoot(String)}
		and {@link #setHome(String)} to initialise it.
	*/
	public PathConverter()
	{}
	
	/**
		Create a new initialised path converter.
		
		@param root The path to the root directory (/)
		@param home The path to the home directory (~)
	*/
	public PathConverter(String root, String home)
	{
		this.home = home;
		this.root = root;
	}
	
	/*
		From the ValueConverter interface
		----------------------------------
	*/
	@Override
	public String convert(String in, Method method) 
	{
		if (!method.isAnnotationPresent(PathSetter.class)) return in;
		if (root != null && in.startsWith("/"))
		{
			in = root + in;
		}
		else if (home != null && in.startsWith("~"))
		{
			in = home + in.substring(1);
		}
		// Remove the 'root' part if the path should be context relative
		PathType pathType = method.getAnnotation(PathSetter.class).pathType();
		if (pathType == PathType.CONTEXT_RELATIVE)
		{
			if (in.startsWith(root)) in = in.substring(root.length());
		}
		return in;		
	}
	// ---------------------------------------

	/**
		Set the value of the root path.
	*/
	public void setRoot(String root)
	{
		this.root = root;
	}
	/**
		Get the value of the root path.
	*/
	public String getRoot()
	{
		return root;
	}

	
	/**
		Set the value of the home path.
	*/
	public void setHome(String home)
	{
		this.home = home;
	}
	
	/**
		Get the value of the home path.
	*/
	public String getHome()
	{
		return home;
	}
	
}
